<?php
/**
 * Admin privacy module
 *
 * @category  Prestashop
 * @category  Module
 * @author    Samdha <contact@samdha.net>
 * @copyright Samdha
 * @license   commercial license see license.txt
 * @version   1.5.0
 *
 * @link      http://scripts.sil.org/OFL logo license
 * @link      http://fontawesome.io - Font Awesome by Dave Gandy - logo author
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

if (!class_exists('sd_addonsblock', false)) {
    if (!defined('_PS_MODULE_DIR_')) { /* PS 1.0 */
        define('_PS_MODULE_DIR_', _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'modules'.DIRECTORY_SEPARATOR);
    }
    if (!defined('_PS_USE_SQL_SLAVE_')) { /* PS 1.3 */
        define('_PS_USE_SQL_SLAVE_', false);
    }

    /**
     * Autoloader for this module classes
     */
    function sdAddonsblockAutoload($class_name)
    {
        $module_name = 'sd_addonsblock';
        $class_name = ltrim($class_name, '\\');
        $file_name  = '';
        $namespace = '';
        if ($last_ns_post = strrpos($class_name, '\\')) {
            $namespace = Tools::substr($class_name, 0, $last_ns_post);
            $class_name = Tools::substr($class_name, $last_ns_post + 1);
            $file_name  = str_replace('\\', DIRECTORY_SEPARATOR, $namespace).DIRECTORY_SEPARATOR;
        }
        $file_name .= str_replace('_', DIRECTORY_SEPARATOR, $class_name).'.php';
        $file_name = _PS_MODULE_DIR_.$module_name.DIRECTORY_SEPARATOR.'classes'.DIRECTORY_SEPARATOR.$file_name;
        if ((Tools::strtolower(basename($file_name)) != 'index.php') && file_exists($file_name)) {
            return require_once($file_name);
        } elseif (version_compare(_PS_VERSION_, '1.4.0.0', '>=') && function_exists('__autoload')) {
            return __autoload($class_name);
        } elseif (version_compare(_PS_VERSION_, '1.4.0.0', '<')
            && is_readable(_PS_ROOT_DIR_.'/classes/'.$class_name.'.php')
        ) {
            require_once _PS_ROOT_DIR_.'/classes/'.$class_name.'.php';
        }
    }
    spl_autoload_register('sdAddonsblockAutoload');

    /**
     * obviously this is not an official module so a big warning will appears
     * if you try to install it on Prestashop 1.6.0.9+
     * this remove the warning
     */
    function sdAddonsblockTrusted()
    {
        if (defined('Module::CACHE_FILE_UNTRUSTED_MODULES_LIST')) {
            $module_name = 'sd_addonsblock';

            // add module as trusted
            $xml_file = _PS_ROOT_DIR_.'/config/xml/themes/'.$module_name.'.xml';
            $xml_content = '<?xml version="1.0"?><modules_list><modules><module action="install" name="'
                .$module_name.'"/></modules></modules_list>';
            $context = Context::getContext();
            if (isset($context->controller)
                && is_subclass_of($context->controller, 'AdminController')
                && $context->controller->controller_name == 'AdminModules'
                && !file_exists($xml_file)
            ) {
                file_put_contents($xml_file, $xml_content);
                Module::generateTrustedXml();
            }
        }
    }
    sdAddonsblockTrusted();


    /* Let's begin */
    class Sd_AddonsBlock extends Samdha_Commons_Module
    {
        public $short_name = 'addonsblock';
        public $config_global = true;

        public function __construct()
        {
            $this->name = 'sd_addonsblock';
            $this->tab = version_compare(_PS_VERSION_, '1.4.0.0', '<')?'Tools':'administration';
            $this->version = '1.5.0';
            $this->module_key = '';

            parent::__construct();

            /* The parent construct is required for translations */
            $this->page = basename(__FILE__, '.php');
            $this->displayName = $this->l('Addons Block');
            $this->description = $this->l('Remove ads from your administration panel');
        }

        /**
         * module installation
         */
        public function install()
        {
            if (!parent::install()
                || !$this->registerHook('displayBackOfficeHeader')) {
                return false;
            }
            return true;
        }

        public function displayForm($token)
        {
            $gamification = Module::getInstanceByName('gamification');

            $tabs = array(
                array('href' => '#tabParameters', 'display_name' => $this->l('Parameters'))
            );

            $this->smarty->assign(array(
                'tabs'         => $tabs,
                'gamification' => Validate::isLoadedObject($gamification)
            ));

            return parent::displayForm($token);
        }

        /**
        * set default config
        **/
        public function getDefaultConfig()
        {
            $result = array(
                'musthave' => 1,
                'tabmodule' => 1,
                'gamification' => 1,
                'theme' => 1,
            );

            return $result;
        }

        public function hookDisplayBackOfficeHeader()
        {
            // Must have modules
            $cache_file = _PS_ROOT_DIR_;
            if (defined('Module::CACHE_FILE_MUST_HAVE_MODULES_LIST')) {
                $cache_file .= Module::CACHE_FILE_MUST_HAVE_MODULES_LIST;
            } else {
                $cache_file .= '/config/xml/must_have_modules_list.xml';
            }
            $empty_file = '<?xml version="1.0" encoding="UTF-8"?>'.PHP_EOL
                .'<modules></modules>'.PHP_EOL;
            if ($this->config->musthave) {
                if (!$this->context->controller->isFresh($cache_file, 60 * 24 * 2)
                    || ($empty_file != Tools::file_get_contents($cache_file))
                ) {
                    file_put_contents($cache_file, $empty_file);
                }
            } else {
                if ($empty_file == Tools::file_get_contents($cache_file)) {
                    unlink($cache_file);
                }
            }

            // Recommended modules
            $cache_file = _PS_ROOT_DIR_;
            if (defined('Module::CACHE_FILE_TAB_MODULES_LIST')) {
                $cache_file .= Module::CACHE_FILE_TAB_MODULES_LIST;
            } else {
                $cache_file .= '/config/xml/tab_modules_list.xml';
            }
            $empty_file = '<?xml version="1.0" encoding="UTF-8"?>'.PHP_EOL
                .'<tab_modules_list></tab_modules_list>'.PHP_EOL;
            if ($this->config->tabmodule) {
                if (!$this->context->controller->isFresh($cache_file, 60 * 24 * 2)
                    || ($empty_file != Tools::file_get_contents($cache_file))
                ) {
                    file_put_contents($cache_file, $empty_file);
                }
            } else {
                if ($empty_file == Tools::file_get_contents($cache_file)) {
                    unlink($cache_file);
                }
            }

            // Gamification ads
            $gamification = Module::getInstanceByName('gamification');
            if (Validate::isLoadedObject($gamification) && $gamification->active) {
                if ($this->config->gamification) {
                    Db::getInstance()->delete(_DB_PREFIX_.'advice');
                    Db::getInstance()->delete(_DB_PREFIX_.'advice_lang');
                    Db::getInstance()->delete(_DB_PREFIX_.'condition_advice');
                    Db::getInstance()->delete(_DB_PREFIX_.'tab_advice');
                }
            }

            // Addons themes on Themes tab
            if ($this->config->theme && ($this->context->controller instanceof AdminThemesController)) {
                $this->context->smarty->registerFilter('output', array($this, 'smartyOutputFilterRemoveThemesBlock'));
            }
        }

        public function smartyOutputFilterRemoveThemesBlock($output)
        {
            return preg_replace('| id="prestastore-content".*?</div>|ms', ' style="display:none" />', $output);
        }
    }
}
